/* Demo - Accelerating Data Warehouse Queries with SQL Server 2012 Columnstore Indexes */
/* Demo was written using 4 logical processors and parallelism enabled (in order to show batch processing) */

-- **
-- Demo starts here...
-- **

USE AdventureWorksDW2012;
GO
-- Let's view CPU, Elapsed time, I/O and the actual execution plan
SET STATISTICS TIME ON;
SET STATISTICS IO ON;

-- Execute this query twice, and measure the second attempt (get past data caching overhead)
SELECT  p.EnglishProductName,
		d.WeekNumberOfYear,
		d.CalendarYear,
		AVG(fpi.UnitCost),
		SUM(fpi.UnitsOut)
FROM	dbo.FactProductInventory as fpi
INNER JOIN dbo.DimProduct as p ON
	fpi.ProductKey = p.ProductKey
INNER JOIN dbo.DimDate as d ON
	fpi.DateKey = d.FullDateAlternateKey
GROUP BY p.EnglishProductName,
		d.WeekNumberOfYear,
		d.CalendarYear
ORDER BY p.EnglishProductName,
		 d.CalendarYear,
		 d.WeekNumberOfYear;

DBCC DROPCLEANBUFFERS

-- What did you see?

SET STATISTICS TIME OFF;
SET STATISTICS IO OFF;


-- Now we'll create the columnstore index
CREATE NONCLUSTERED COLUMNSTORE INDEX [IX_CS_FactProductInventory]
ON dbo.FactProductInventory
( 
		ProductKey, 
		DateKey, 
		UnitCost, 
		UnitsIn, 
		UnitsOut, 
		UnitsBalance
);


-- Let's view CPU, Elapsed time, I/O and the actual execution plan
SET STATISTICS TIME ON;
SET STATISTICS IO ON;

DBCC DROPCLEANBUFFERS

-- Execute this query twice, and measure the second attempt (get past data caching overhead)
SELECT  p.EnglishProductName,
		d.WeekNumberOfYear,
		d.CalendarYear,
		AVG(fpi.UnitCost),
		SUM(fpi.UnitsOut)
FROM	dbo.FactProductInventory as fpi
INNER JOIN dbo.DimProduct as p ON
	fpi.ProductKey = p.ProductKey
INNER JOIN dbo.DimDate as d ON
	fpi.DateKey = d.FullDateAlternateKey
GROUP BY p.EnglishProductName,
		d.WeekNumberOfYear,
		d.CalendarYear
ORDER BY p.EnglishProductName,
		 d.CalendarYear,
		 d.WeekNumberOfYear;

-- What did you see?

SET STATISTICS TIME OFF;
SET STATISTICS IO OFF;

-- Now let's demonstrate updating a CS table with partitioning
-- Removing CS index for now and supporting NCI created earlier
DROP INDEX [IX_CS_FactProductInventory] ON [dbo].[FactProductInventory];
GO

DROP INDEX [NCI_FactProductInventory_DateKey] ON [dbo].[FactProductInventory];
GO

-- Create partition function and scheme
CREATE PARTITION FUNCTION [pfn_DateKey](date) AS RANGE RIGHT 
FOR VALUES (N'2005-01-13', N'2005-02-13', N'2005-03-13', N'2005-04-13', N'2005-05-13', N'2005-06-13', N'2005-07-13',
 N'2005-08-13', N'2005-09-13', N'2005-10-13', N'2005-11-13', N'2005-12-13', N'2006-01-13', N'2006-02-13',
  N'2006-03-13', N'2006-04-13', N'2006-05-13', N'2006-06-13', N'2006-07-13', N'2006-08-13', N'2006-09-13',
   N'2006-10-13', N'2006-11-13', N'2006-12-13', N'2007-01-13', N'2007-02-13', N'2007-03-13', N'2007-04-13', 
   N'2007-05-13', N'2007-06-13', N'2007-07-13', N'2007-08-13', N'2007-09-13', N'2007-10-13', N'2007-11-13', 
   N'2007-12-13', N'2008-01-13', N'2008-02-13', N'2008-03-13', N'2008-04-13', N'2008-05-13', N'2008-06-13',
    N'2008-07-13', N'2008-08-13', N'2008-09-13', N'2008-10-13', N'2008-11-13', N'2008-12-13');
GO

CREATE PARTITION SCHEME [pfn_DateKey] AS PARTITION [pfn_DateKey] 
ALL TO([PRIMARY]);
GO

CREATE CLUSTERED INDEX [CI_DateKey] 
ON [dbo].[FactProductInventory]
	([DateKey], [ProductKey]) ON [pfn_DateKey]([DateKey]);
GO

CREATE NONCLUSTERED INDEX [NCI_UnitCost] 
ON [dbo].[FactProductInventory]
	([UnitCost] ASC)
 ON [pfn_DateKey]([DateKey]);
GO

-- Add in the CS index (notice no need to specify the partition scheme)
CREATE NONCLUSTERED COLUMNSTORE INDEX [NSI_FactProductInventory] 
ON [dbo].[FactProductInventory]
(
	[ProductKey],
	[DateKey],
	[UnitCost],
	[UnitsIn],
	[UnitsOut],
	[UnitsBalance]
);
GO

-- Demo that updates still aren't possible
INSERT dbo.FactProductInventory
(ProductKey, DateKey, UnitCost, UnitsIn, UnitsOut, UnitsBalance)
VALUES (167, '2005-01-14', .28, 0, 0, 870);
GO

-- Now we'll create a new archive table
CREATE TABLE [dbo].[Archive_FactProductInventory](
	[ProductKey] [int] NOT NULL,
	[UnitCost] [money] NOT NULL,
	[UnitsIn] [int] NOT NULL,
	[UnitsOut] [int] NOT NULL,
	[UnitsBalance] [int] NOT NULL,
	[DateKey] [date] NOT NULL
) ON [PRIMARY];
GO

CREATE CLUSTERED INDEX [Archive_FactProductInventory_CI_DateKey] ON [dbo].[Archive_FactProductInventory]
(
	[DateKey] ASC,
	[ProductKey] ASC
)ON [PRIMARY];
GO

CREATE NONCLUSTERED INDEX [Archive_FactProductInventory_NCI_UnitCost] ON [dbo].[Archive_FactProductInventory]
([UnitCost] ASC)
INCLUDE ( 	[DateKey])  ON [PRIMARY];
GO

ALTER TABLE [dbo].[Archive_FactProductInventory]  
WITH CHECK ADD  CONSTRAINT [Archive_FactProductInventory_FK_ProductInventory_DimProduct] 
FOREIGN KEY([ProductKey])
REFERENCES [dbo].[DimProduct] ([ProductKey]);
GO

ALTER TABLE [dbo].[Archive_FactProductInventory] 
 WITH CHECK ADD  CONSTRAINT [chk_Archive_FactProductInventory]
  CHECK  ([DateKey]>=N'2006-06-13' AND [DateKey]<N'2006-07-13');
  GO

-- 6,210,288 rows
SELECT COUNT(*) FROM [AdventureWorksDW2012].[dbo].[FactProductInventory];

-- 0 rows
SELECT COUNT(*) FROM [AdventureWorksDW2012].[dbo].[Archive_FactProductInventory];

-- Switch out a partition
ALTER TABLE [AdventureWorksDW2012].[dbo].[FactProductInventory]
 SWITCH PARTITION 19 TO [AdventureWorksDW2012].[dbo].[Archive_FactProductInventory];
GO

-- Validate that rows were moved to the archive table
-- 145,440 now in the archive table
SELECT COUNT(*)
FROM [AdventureWorksDW2012].[dbo].[Archive_FactProductInventory];
GO

-- Validate that this happened while normal updates (not partitioned ones, are disallowed)
INSERT dbo.FactProductInventory
(ProductKey, DateKey, UnitCost, UnitsIn, UnitsOut, UnitsBalance)
VALUES (167, '2005-01-14', .28, 0, 0, 870);
GO

-- We can also switch in back data to the main table

-- Should be 6,064,848
SELECT COUNT(*) FROM [dbo].[FactProductInventory];
GO

-- Let's create a CS index on the archive table (you'll see an error if you do not)
CREATE NONCLUSTERED COLUMNSTORE INDEX [NSI_ArchiveFactProductInventory] 
ON [dbo].[Archive_FactProductInventory]
(
	[ProductKey],
	[DateKey],
	[UnitCost],
	[UnitsIn],
	[UnitsOut],
	[UnitsBalance]
)WITH (DROP_EXISTING = OFF);
GO

ALTER TABLE [dbo].[Archive_FactProductInventory] 
SWITCH  TO [dbo].[FactProductInventory]
PARTITION 19;
GO

-- Now check the rowcounts of both tables (it worked!)
-- back to 6,210,288
SELECT COUNT(*) FROM [dbo].[FactProductInventory];
GO








